(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('redux')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'redux'], factory) :
  (global = global || self, factory(global.pippo = {}, global.React, global.redux));
}(this, function (exports, React, redux) { 'use strict';

  var React__default = 'default' in React ? React['default'] : React;

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);

    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      if (enumerableOnly) symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
      keys.push.apply(keys, symbols);
    }

    return keys;
  }

  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};

      if (i % 2) {
        ownKeys(source, true).forEach(function (key) {
          _defineProperty(target, key, source[key]);
        });
      } else if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
      } else {
        ownKeys(source).forEach(function (key) {
          Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
      }
    }

    return target;
  }

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
  }

  function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
  }

  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

      return arr2;
    }
  }

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArray(iter) {
    if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
  }

  function _iterableToArrayLimit(arr, i) {
    if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
      return;
    }

    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance");
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }

  var StoreContext = React__default.createContext(null);

  var SET_VALUE = 'SET VALUE';
  var REMOVE_VALUE = 'REMOVE VALUE';

  function mainReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments.length > 1 ? arguments[1] : undefined;

    switch (action.type) {
      case SET_VALUE:
        {
          var _action$payload = action.payload,
              key = _action$payload.key,
              value = _action$payload.value;

          var newState = _objectSpread2({}, state);

          newState[key] = value;
          return newState;
        }

      case REMOVE_VALUE:
        {
          var _key = action.payload.key;

          var _newState = _objectSpread2({}, state);

          delete _newState[_key];
          return _newState;
        }

      default:
        return state;
    }
  }

  var rootReducer = redux.combineReducers({
    main: mainReducer
  });

  var logger = function logger(_ref) {
    var getState = _ref.getState;
    return function (next) {
      return function (action) {
        var prevState = getState().main;
        var returnValue = next(action);
        console.group("%c react-context-hook", 'font-weight: bold');
        console.info("%c ".concat('PREV STATE', " "), 'background: #00e5a0;font-weight: bold', prevState);
        console.info("%c ".concat('ACTION', " %c ", action.type), 'background: yellow;font-weight: bold', 'font-weight: bold;', action.payload.key, action.payload.value);
        console.info("%c ".concat('NEXT STATE', " "), 'background: #00cbff;font-weight: bold', getState().main);
        console.groupEnd();
        return returnValue;
      };
    };
  };
  var listernerMiddleware = function listernerMiddleware(listener) {
    return function (_ref2) {
      var getState = _ref2.getState;
      return function (next) {
        return function (action) {
          var value = next(action);
          listener(getState().main);
          return value;
        };
      };
    };
  };

  function createStoreCreator() {
    var initialValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var config = arguments.length > 1 ? arguments[1] : undefined;
    var logging = config.logging,
        listener = config.listener;
    return redux.createStore(rootReducer, {
      main: initialValue
    }, redux.applyMiddleware.apply(void 0, _toConsumableArray([listener && listernerMiddleware(listener), logging && logger].filter(Boolean))));
  }

  var defaultConfig = {};

  /**
   * @param {ReactElement} WrappedComponent the component to connect with the store
   * @param {Object} initialValue the initial store value or nothing
   * @param {Object} config the custom configuration. If nothing is passed will use the default config
   * @param {Function} config.listener a function that is triggered each time the global state is modified.
   * @param {boolean} config.logging - default `false` - if true it will log changes to console
   * @example
   *const initialState = { count: 10 }
   *
   *const storeConfig = {
   *  listener: state => {
   *    console.log('state changed', state)
   *  },
   *  logging: process.env.NODE_ENV !== 'production'
   *}
   *
   *export default withStore(App, initialState, storeConfig)
   */

  var withStore = function withStore(WrappedComponent, initialValue, config) {
    var userConfig = Object.freeze(_objectSpread2({}, defaultConfig, {}, config));
    var store = createStoreCreator(initialValue, userConfig);
    return function (props) {
      return React__default.createElement(StoreContext.Provider, {
        value: {
          store: store
        }
      }, React__default.createElement(WrappedComponent, props));
    };
  };

  var Subscription =
  /*#__PURE__*/
  function () {
    function Subscription(store, onStateChange) {
      _classCallCheck(this, Subscription);

      this.store = store; // this.handleChangeWrapper = this.handleChangeWrapper.bind(this)

      this.unsuscribeFromSubscription = null;
      this.onStateChange = onStateChange.bind(this);
    }

    _createClass(Subscription, [{
      key: "subscribe",
      value: function subscribe() {
        this.unsuscribeFromSubscription = this.store.subscribe(this.onStateChange);
      }
    }, {
      key: "unsubscribe",
      value: function unsubscribe() {
        this.unsuscribeFromSubscription();
        this.unsuscribeFromSubscription = null;
      }
    }]);

    return Subscription;
  }();

  var setStoreValueAction = function setStoreValueAction(key, value) {
    return {
      type: SET_VALUE,
      payload: {
        key: key,
        value: value
      }
    };
  };
  var deleteStoreValueAction = function deleteStoreValueAction(key) {
    return {
      type: REMOVE_VALUE,
      payload: {
        key: key
      }
    };
  };

  function equalityFn(a, b) {
    return a === b;
  }

  function useSelector(stateSelector) {
    var selectedState = useSelectorWithStoreAndSubscription(stateSelector);
    return selectedState;
  }

  function useSelectorWithStoreAndSubscription(stateSelectorFn) {
    var _useReducer = React.useReducer(function (s) {
      return s + 1;
    }, 0),
        _useReducer2 = _slicedToArray(_useReducer, 2),
        forceRender = _useReducer2[1];

    var _useContext = React.useContext(StoreContext),
        store = _useContext.store;

    var latestStateSelectorFn = React.useRef();
    var latestSelectedState = React.useRef();

    function checkForUpdates() {
      try {
        var newSelectedState = latestStateSelectorFn.current(store.getState().main);

        if (equalityFn(newSelectedState, latestSelectedState.current)) {
          return;
        }

        latestSelectedState.current = newSelectedState;
      } catch (err) {// ignore all errors
      }

      forceRender({});
    }

    var subscription = React.useMemo(function () {
      return new Subscription(store, checkForUpdates);
    }, [store, checkForUpdates]);
    var resultValue;

    if (stateSelectorFn !== latestStateSelectorFn.current) {
      resultValue = stateSelectorFn(store.getState().main);
    } else {
      resultValue = latestSelectedState.current;
    }

    React.useEffect(function () {
      latestStateSelectorFn.current = stateSelectorFn;
      latestSelectedState.current = resultValue;
    });
    React.useEffect(function () {
      subscription.subscribe();
      checkForUpdates();
      return function () {
        return subscription.unsubscribe();
      };
    }, [store, subscription]);
    return resultValue;
  }
  /**
   * `useStore` is a React Hook that access a value stored in the application global store. It returns the value, a function to update it (like React.useState) and a function to delete it.
   *
   * @param {string} key - The lookup key to find the saved value in the store
   * @param {any} defaultValue - The value if the value in the store is missing
   *
   * @returns {array} an array with length 3:<br>
   * position 0 - the value of the data in the store.<br>
   * position 1 - a function *setValue* to modify the data in the store. When used, this function return a promise that resolve nothing, thus you can use `setValue('a value').then(() => {doSomething() //when the store did update})`<br>
   * position 2 - a function *deleteValue* to delete the value from the store. When used, this function return a promise that resolve nothing, thus you can use `deleteValue('a value').then(() => {doSomething() //when the store did update})`
   *
   * @example
   * import {useStore} from 'react-context-hook'
   * const [username, setUsername, deleteUsername] = useStore('username')
   * <div>hello {username}</div>
   * <button onClick={()=> setUsername('my_username')}>set username</button>
   *
   */


  function useStore(key, defaultValue) {
    return [useStoreValue(key, defaultValue), useSetStoreValue(key), useDeleteStoreValue(key)];
  }
  /**
   * Returns a function to set or update a variable in the store. You want to use this hook when you just need to modify the store, not read or delete a value from it.
   * @param {string} key - the name of the variable to set in the store
   * @returns {Function} - a function to set a variable in the store with the given name When used, this function return a promise that resolve nothing, thus you can use `setValue('a value').then(() => {doSomething() //when the store did update})`
   *
   * @example
   * import {useSetStoreValue} from 'react-context-hook'
   * const setUsername = useSetStoreValue('username')
   * <button onClick={()=> setUsername('my_username')}>set username</button>
   */


  function useSetStoreValue(key) {
    var _useContext2 = React.useContext(StoreContext),
        store = _useContext2.store;

    return function (value) {
      store.dispatch(setStoreValueAction(key, value));
    };
  }
  /**
   * Returns a function to delete a variable in the store. You want to use this hook when you just need to delete a value in the store, not read or set a value from it.
   * @param {string} key - the name of the variable to set in the store
   * @returns {Function} - a function to delete a variable in the store with the given name. When used, this function return a promise that resolve nothing, thus you can use `deleteValue('a value').then(() => {doSomething() //when the store did update})`
   *
   * @example
   * import {useDeleteStoreValue} from 'react-context-hook'
   * const deleteUsername = useDeleteStoreValue('username')
   * <button onClick={()=> deleteUsername('my_username')}>set username</button>
   */


  function useDeleteStoreValue(key) {
    var _useContext3 = React.useContext(StoreContext),
        store = _useContext3.store;

    return function (value) {
      store.dispatch(deleteStoreValueAction(key));
    };
  }
  /**
   *This React hook returns an array to read and modify a value in the store:
   * `const [value, setValue] = useGetAndSet('a_lookup_key_in_the_store')`. The name of the variable in the arry is arbitrary and you can choose any string you like.
   * @param {string} key - The lookup key to find the saved value in the store
   * @param {any} defaultValue - The default value if missing
   * @returns {array} an array with length 2:<br>
   * position 0 - the value of the data in the store.<br>
   * position 1 - a function *setValue* to modify the data in the store. When used, this function return a promise that resolve nothing, thus you can use `setValue('a value').then(() => {doSomething() //when the store did update})`<br>
   *
   * @example
   * import {useGetAndSet} from 'react-context-hook'
   * const [username, setUsername] = useGetAndSet('username')
   * <div>hello {username}</div>
   * <button onClick={()=> setUsername('my_username')}>set username</button>
   *
   *  const [value, setValue] = useGetAndSet('a_lookup_key_in_the_store')
   *
   */


  function useGetAndSet(key, defaultValue) {
    return [useStoreValue(key, defaultValue), useSetStoreValue(key)];
  }
  /**
   *This React hook returns an array to read and delete a value in the store:
   * `const [value, deleteValue] = useGetAndDelete('a_lookup_key_in_the_store')`. The name of the variable in the arry is arbitrary and you can choose any string you like.
   * @param {string} key - The lookup key to find the saved value in the store
   *
   * @returns {array} an array with length 2:<br>
   * position 0 - the value of the data in the store.<br>
   * position 1 - a function *deleteValue* to delete the data in the store. When used, this function return a promise that resolve nothing, thus you can use `deleteValue('a value').then(() => {doSomething() //when the store did update})`<br>
   *
   * @example
   * import {useGetAndDelete} from 'react-context-hook'
   * const [username, deleteUsername] = useGetAndDelete('username')
   * <div>hello {username}</div>
   * <button onClick={()=> deleteUsername('my_username')}>set username</button>
   *
   */


  function useGetAndDelete(key) {
    return [useStoreValue(key), useDeleteStoreValue(key)];
  }
  /**
   *This React hook returns an array to set and delete a value in the store:
   * `const [setValue, deleteValue] = useGetAndDelete('a_lookup_key_in_the_store')`. The name of the variable in the arry is arbitrary and you can choose any string you like.
   * @param {string} key - The lookup key to find the saved value in the store
   *
   * @returns {array} an array with length 2:<br>
   * position 0 - a function *setValue* to modify the data in the store. When used, this function return a promise that resolve nothing, thus you can use `setValue('a value').then(() => {doSomething() //when the store did update})`<br>
   * position 1 - a function *deleteValue* to delete the data in the store. When used, this function return a promise that resolve nothing, thus you can use `deleteValue('a value').then(() => {doSomething() //when the store did update})`<br>
   *
   * @example
   * import {useGetAndDelete} from 'react-context-hook'
   * const [username, deleteUsername] = useGetAndDelete('username')
   * <div>hello {username}</div>
   * <button onClick={()=> deleteUsername('my_username')}>set username</button>
   *
   */


  function useSetAndDelete(key) {
    return [useSetStoreValue(key), useDeleteStoreValue(key)];
  }
  /**
   *
   * @param {string} key - the name of the variable / value to be retrieved in the global store.
   * @param {any} [defaultValue] - an optional default value, if the value in the global store is not present.
   *
   * @returns {any} the value on the global store, or the default value if passed, or `undefined`
   */


  function useStoreValue(key, defaultValue) {
    function stateSelector(state) {
      return state[key];
    }

    return useSelector(stateSelector) || defaultValue;
  }
  /**
   * Returns the whole store value, with all the variables stored in it. Changes to this object will not change the store
   *
   * @returns {object} - An object representing the whole store value in read only mode.
   *
   * @example
   * import {useStoreState} from 'react-context-hook'
   * const store = useStoreState()
   * console.log('the store is', JSON.stringify(store))
   */


  function useStoreState() {
    function stateSelector(state) {
      return state;
    }

    return useSelector(stateSelector);
  }

  exports.useDeleteStoreValue = useDeleteStoreValue;
  exports.useGetAndDelete = useGetAndDelete;
  exports.useGetAndSet = useGetAndSet;
  exports.useSetAndDelete = useSetAndDelete;
  exports.useSetStoreValue = useSetStoreValue;
  exports.useStore = useStore;
  exports.useStoreState = useStoreState;
  exports.useStoreValue = useStoreValue;
  exports.withStore = withStore;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.es.js.umd.js.map
